@@ -65,6 +65,61 @@ typedef struct {
 	void *ss;
 } SnmpObject;
 
+/* Helper function: convert a tuple to an OID. */
+static int
+tuple2oid(PyObject *tuple, oid anOID[], size_t *anOID_len) {
+	size_t len;
+	int i;
+	PyObject *poid;
+	if (!PyTuple_Check(tuple)) {
+		PyErr_Format(PyExc_TypeError,
+			     "OID should be a tuple of integers");
+		return -1;
+	}
+	len = PyTuple_Size(tuple);
+	if (len > *anOID_len) {
+		PyErr_Format(PyExc_ValueError,
+			     "OID is too large: %zd > %zd",
+			     len, *anOID_len);
+		return -1;
+	}
+	for (i = 0; i < len; i++) {
+		if ((poid = PyTuple_GetItem(tuple, i)) == NULL)
+			return -1;
+		if (PyLong_Check(poid))
+			anOID[i] = (oid)PyLong_AsUnsignedLong(poid);
+		else if (PyInt_Check(poid))
+			anOID[i] = (oid)PyInt_AsLong(poid);
+		else {
+			PyErr_Format(PyExc_TypeError,
+				     "element %d of OID is not an integer",
+				     i);
+			return -1;
+		}
+	}
+	*anOID_len = len;
+	return 0;
+}
+
+/* Helper function: convert an OID to a tuple. */
+static PyObject*
+oid2tuple(oid anOID[], size_t anOID_len) {
+	PyObject *resultoid;
+	PyObject *tmp;
+	int i;
+	if ((resultoid = PyTuple_New(anOID_len)) == NULL)
+		return NULL;
+	for (i = 0; i < anOID_len; i++) {
+		if ((tmp = PyLong_FromLong(anOID[i])) == NULL) {
+			Py_DECREF(resultoid);
+			return NULL;
+		}
+		PyTuple_SetItem(resultoid, i, tmp);
+	}
+	return resultoid;
+}
+
+/* SNMP sessions */
 static void
 Snmp_dealloc(SnmpObject* self)
 {
@@ -105,7 +160,7 @@ Snmp_init(SnmpObject *self, PyObject *args, PyObject *kwds)
 		*authprotocol=NULL, *authpassword=NULL,
 		*privprotocol=NULL, *privpassword=NULL;
 	char *chost=NULL;
-	int version = 2;
+	int version = SNMP_DEFAULT_VERSION;
 	struct snmp_session session;
 
 	static char *kwlist[] = {"host", "community", "version",
@@ -136,6 +191,10 @@ Snmp_init(SnmpObject *self, PyObject *args, PyObject *kwds)
 	case 3:
 		session.version = SNMP_VERSION_3;
 		break;
+	case SNMP_DEFAULT_VERSION:
+		session.version = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
+						     NETSNMP_DS_LIB_SNMPVERSION);
+		break;
 	default:
 		PyErr_Format(PyExc_ValueError, "invalid SNMP version: %d",
 		    version);
@@ -273,11 +332,9 @@ Snmp_repr(SnmpObject *self)
 		Py_DECREF(peer);
 		return NULL;
 	}
-	result = PyString_FromFormat("%s(host=%s, version=%d)",
+	result = PyString_FromFormat("%s(host=%s)",
 	    self->ob_type->tp_name,
-	    PyString_AsString(rpeer),
-	    (sptr->version == SNMP_VERSION_1)?1:
-				     ((sptr->version == SNMP_VERSION_2c)?2:3));
+	    PyString_AsString(rpeer));
 	Py_DECREF(rpeer);
 	Py_DECREF(peer);
 	return result;
@@ -320,15 +377,63 @@ converterr:
 	return NULL;
 }
 
+static int
+Snmp_handle_answer(int op,
+		   netsnmp_session * session,
+		   int reqid, netsnmp_pdu *pdu, void *magic)
+{
+	struct synch_state **state = (struct synch_state **) magic;
+
+	if (*state == NULL) {
+		free(state);
+		return 0;
+	}
+	(*state)->waiting = 0;
+
+	if (op == NETSNMP_CALLBACK_OP_RECEIVED_MESSAGE && pdu) {
+		switch (pdu->command) {
+		case SNMP_MSG_RESPONSE:
+			(*state)->pdu = snmp_clone_pdu(pdu);
+			(*state)->status = STAT_SUCCESS;
+			session->s_snmp_errno = SNMPERR_SUCCESS;
+			free(state);
+			return 1;
+		}
+	} else if (op == NETSNMP_CALLBACK_OP_TIMED_OUT) {
+		(*state)->pdu = NULL;
+		(*state)->status = STAT_TIMEOUT;
+		session->s_snmp_errno = SNMPERR_TIMEOUT;
+		SET_SNMP_ERROR(SNMPERR_TIMEOUT);
+		free(state);
+		return 1;
+	} else if (op == NETSNMP_CALLBACK_OP_DISCONNECT) {
+		(*state)->pdu = NULL;
+		(*state)->status = STAT_ERROR;
+		session->s_snmp_errno = SNMPERR_ABORT;
+		SET_SNMP_ERROR(SNMPERR_ABORT);
+		free(state);
+		return 1;
+	}
+	/* Unexpected error. */
+	(*state)->pdu = NULL;
+	(*state)->status = STAT_ERROR;
+	session->s_snmp_errno = SNMPERR_PROTOCOL;
+	SET_SNMP_ERROR(SNMPERR_PROTOCOL);
+	snmp_set_detail("Unexpected reply");
+	free(state);
+	return 0;
+}
+
 static PyObject*
 Snmp_op(SnmpObject *self, PyObject *args, int op)
 {
-	PyObject *roid, *poid, *resultvalue=NULL, *resultoid=NULL,
+	PyObject *roid, *resultvalue=NULL, *resultoid=NULL,
 	    *result=NULL, *results=NULL, *tmp, *setobject;
 	struct snmp_pdu *pdu=NULL, *response=NULL;
 	struct variable_list *vars;
+	struct synch_state state, **hstate = NULL;
 	oid anOID[MAX_OID_LEN];
-	size_t anOID_len = MAX_OID_LEN;
+	size_t anOID_len;
 	int i = 0, j = 0, status, type;
 	struct ErrorException *e;
 	long long counter64;
@@ -349,33 +454,9 @@ Snmp_op(SnmpObject *self, PyObject *args, int op)
 	while (j < PyTuple_Size(args)) {
 		if ((roid = PyTuple_GetItem(args, j)) == NULL)
 			goto operror;
-		if (!PyTuple_Check(roid)) {
-			PyErr_Format(PyExc_TypeError,
-			    "argument %d should be a tuple of integers",
-			    j);
+		anOID_len = sizeof(anOID)/sizeof(oid);
+		if (tuple2oid(roid, anOID, &anOID_len) == -1)
 			goto operror;
-		}
-		anOID_len = PyTuple_Size(roid);
-		if (anOID_len > MAX_OID_LEN) {
-			PyErr_Format(PyExc_ValueError,
-			    "OID #%d is too large: %zd > %d",
-			    j, anOID_len, MAX_OID_LEN);
-			goto operror;
-		}
-		for (i = 0; i < anOID_len; i++) {
-			if ((poid = PyTuple_GetItem(roid, i)) == NULL)
-				goto operror;
-			if (PyLong_Check(poid))
-				anOID[i] = (oid)PyLong_AsUnsignedLong(poid);
-			else if (PyInt_Check(poid))
-				anOID[i] = (oid)PyInt_AsLong(poid);
-			else {
-				PyErr_Format(PyExc_TypeError,
-				    "element %d of OID #%d is not an integer",
-				    i, j);
-				goto operror;
-			}
-		}
 		if (op != SNMP_MSG_SET) {
 			snmp_add_null_var(pdu, anOID, anOID_len);
 			j++;
@@ -390,11 +471,54 @@ Snmp_op(SnmpObject *self, PyObject *args, int op)
 			j += 2;
 		}
 	}
-	Py_BEGIN_ALLOW_THREADS
-	status = snmp_sess_synch_response(self->ss, pdu, &response);
-	Py_END_ALLOW_THREADS
-	free(buffer);
-	pdu = NULL;		/* Don't try to free it from now */
+	/* Start sending... */
+	/* We don't use snmp_sess_synch_response because it is not interruptible. */
+	if ((hstate = calloc(1, sizeof(struct synch_state *))) == NULL) {
+		PyErr_NoMemory();
+		goto operror;
+	}
+	/* hstate should be freed by the callback, not here. */
+	memset(&state, 0, sizeof(struct synch_state));
+	*hstate = &state;
+	if (snmp_sess_async_send(self->ss, pdu, Snmp_handle_answer, hstate) == 0) {
+		Snmp_raise_error(self->ss, 0);
+		goto operror;
+	}
+	pdu = NULL;
+	state.waiting = 1;
+	while(state.waiting) {
+		fd_set fdset;
+		int    block  = 1;
+		int    count;
+		int    numfds = 0;
+		struct timeval tv = {0, 0};
+		FD_ZERO(&fdset);
+		snmp_sess_select_info(self->ss, &numfds, &fdset, &tv, &block);
+		Py_BEGIN_ALLOW_THREADS
+		count = select(numfds, &fdset, 0, 0, (block == 1)?NULL:&tv);
+		Py_END_ALLOW_THREADS
+		if (count > 0)
+			snmp_sess_read(self->ss, &fdset);
+		else
+			switch (count) {
+			case 0:
+				snmp_sess_timeout(self->ss);
+				break;
+			case -1:
+				if (errno != EINTR) {
+					PyErr_SetFromErrno(PyExc_IOError);
+					goto operror;
+				}
+				if (PyErr_CheckSignals())
+					/* Ctrl-C */
+					goto operror;
+				break;
+			}
+	}
+	response = state.pdu;
+	status   = state.status;
+	hstate   = NULL;	/* Means: callback has been called */
+	/* Got answer... */
 	if (status != STAT_SUCCESS) {
 		Snmp_raise_error(self->ss, 0);
 		goto operror;
@@ -502,13 +626,8 @@ Snmp_op(SnmpObject *self, PyObject *args, int op)
 		if (resultvalue == NULL) goto operror;
 		
 		/* And now, the OID */
-		if ((resultoid = PyTuple_New(vars->name_length)) == NULL)
+		if ((resultoid = oid2tuple(vars->name, vars->name_length)) == NULL)
 			goto operror;
-		for (i = 0; i < vars->name_length; i++) {
-			if ((tmp = PyLong_FromLong(vars->name[i])) == NULL)
-				goto operror;
-			PyTuple_SetItem(resultoid, i, tmp);
-		}
 		if ((result = PyTuple_Pack(2, resultoid, resultvalue)) == NULL)
 			goto operror;
 		Py_CLEAR(resultoid);
@@ -525,6 +644,8 @@ operror:
 	Py_XDECREF(resultoid);
 	Py_XDECREF(results);
 	Py_XDECREF(result);
+	if (hstate) *hstate = NULL; /* Callback: don't run anything */
+	free(buffer);
 	if (pdu)
 		snmp_free_pdu(pdu);
 	if (response)
@@ -728,10 +849,9 @@ initsnmp(void)
 	}
 
 	/* Constants */
-#define ADDCONSTANT(x)				\
-	if ((c = PyInt_FromLong(x)) == NULL)	\
-		return;				\
-	PyModule_AddObject(m, #x, c)
+#define ADDCONSTANT(x)					\
+	if (PyModule_AddIntConstant(m, #x, x) == -1)	\
+		return;
 	ADDCONSTANT(ASN_BOOLEAN);
 	ADDCONSTANT(ASN_INTEGER);
 	ADDCONSTANT(ASN_UNSIGNED);
